#!/usr/bin/env python
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

π = np.pi

N = 2
d = 10.5
a = 0.
θ = 0.5

nn = 10


def farfield(N, d, a, θ):
    x0 = np.sin(θ * π)
    dx1 = 0.5 / (N * π * d * nn)
    x1 = np.arange(-x0, x0 + 0.5 * dx1, dx1)
    if a > 0:
        dx2 = 0.5 / (π * a * nn)
        x2 = np.arange(-x0, x0 + 0.5 * dx2, dx2)
        x = np.concatenate((x1, x2))
        x.sort(kind='mergesort')
    else:
        x = x1
    print(len(x))
    res = (np.sinc(π * a * x) * np.sin(N * π * d * x) / np.sin(π * d * x))**2
    res[x == 0] = (np.sinc(π * a * x[x == 0]) * N)**2
    res /= max(res)
    return np.arcsin(x) / π, res


fig, ax = plt.subplots()

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Interferencja")
manager.full_screen_toggle()

ax.axvline(0, color='k', lw=0.7)

df = 0.0
dφ = 0.0

tt = np.linspace(0., 100., 10001)

line, = ax.plot(*farfield(N, d, a, θ), lw=2.0)

ax.set_xlabel('$\\theta$ [$\\pi$]')
ax.set_ylabel('Intensywność')

ax.set_xlim(-θ, θ)
ax.set_ylim(-0.01, 1.02)

fig.tight_layout()

fig.subplots_adjust(bottom=0.22)
nax = fig.add_axes([0.25, 0.10, 0.65, 0.03])
dax = fig.add_axes([0.25, 0.07, 0.65, 0.03])
aax = fig.add_axes([0.25, 0.04, 0.65, 0.03])
θax = fig.add_axes([0.25, 0.01, 0.65, 0.03])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

N_slider = Slider(ax=nax, label='Ilość szczelin', valmin=1, valmax=100, valinit=N, valstep=1, **sst)
d_slider = Slider(ax=dax, label='$d$ [$\\lambda$]', valmin=0.125, valmax=50, valinit=0.5, valstep=0.125, **sst)
a_slider = Slider(ax=aax, label='$a$ [$\\lambda$]', valmin=0., valmax=10, valinit=a, valstep=0.125 / 4, **sst)
θ_slider = Slider(ax=θax, label='Zakres [$\\pi]$', valmin=0.05, valmax=0.5, valinit=θ, valstep=0.05, **sst)

d_slider.set_val(d)


def on_slider(val):
    line.set_data(*farfield(N_slider.val, d_slider.val, a_slider.val, θ_slider.val))
    ax.set_xlim(-θ_slider.val, θ_slider.val)
    fig.canvas.draw_idle()


N_slider.on_changed(on_slider)
d_slider.on_changed(on_slider)
a_slider.on_changed(on_slider)
θ_slider.on_changed(on_slider)

plt.show()
